#include <gint/display.h>

#include "player.h"
#include "conf.h"
#include "camera.h"
#include "input.h"
#include "collide.h"

void player_move(Player *player, const Level *level) {
}

void player_step(Player *player, Input *input, const Level *level) {
	Vec move = {
		(input_is_down(input, K_RIGHT) - input_is_down(input, K_LEFT)),
		(input_is_down(input, K_DOWN) - input_is_down(input, K_UP))
	};
	vec_mul(&move, 32 * PXS); /* set speed */
	Vec destination;
	vec_cpy(&destination, player->pos);
	vec_add(&destination, move);
	if (collide_full(player, player->pos, level, level->solid_layer) ||
		!collide_full(player, destination, level, level->solid_layer))
	{
		vec_cpy(&player->pos, destination);
	}
}

void player_draw(Player *player, Camera *camera) {
	Vec tl;
	Vec br;
	vec_cpy(&tl, player->pos);
	vec_sub(&tl, player->origin);
	vec_cpy(&br, tl);
	vec_div(&tl, VEC_PRECISION);
	vec_div(&br, VEC_PRECISION);
	vec_add(&br, player->vbox);
	vec_sub(&tl, camera->offset);
	vec_sub(&br, camera->offset);
	vec_mul(&tl, SCALE);
	vec_mul(&br, SCALE);
	vec_add(&br, (Vec){SCALE - 1, SCALE - 1});
	vec_drect(tl, br, C_BLACK);
}

void player_draw_debug(Player *player, uint step, const Level *level, uint layer_id) {
	dprint(0, 0, C_BLACK, "x:  %d", player->pos.x);
	dprint(0, 10, C_BLACK, "y:  %d", player->pos.y);
	dprint(0, 20, C_BLACK, "vp: %d", VEC_PRECISION);
	dprint(0, 30, C_BLACK, "st: %u", step);
	dprint(0, 40, C_BLACK, "cx: %d", player->pos.x / TILE_SIZE);
	dprint(0, 50, C_BLACK, "cy: %d", player->pos.y / TILE_SIZE);
	dprint(0, 60, C_BLACK, "cl: %d", collide_point(player->pos, level, layer_id));
}

